@@ -8,6 +8,46 @@
 var hiddenStuffHeight, mainContentHeight, sidebarHeight, mainContentWidth,
     verticalItemHeight, accordion, resizeTimer, cachedScrollbarWidth;
 
+// Csrf-for-ajax fix suggested in
+// https://docs.djangoproject.com/en/1.3/ref/contrib/csrf/#ajax
+$(document).ajaxSend(function(event, xhr, settings) {
+    function getCookie(name) {
+        var cookieValue = null;
+        if (document.cookie && document.cookie != '') {
+            var cookies = document.cookie.split(';');
+            for (var i = 0; i < cookies.length; i++) {
+                var cookie = jQuery.trim(cookies[i]);
+                // Does this cookie string begin with the name we want?
+                if (cookie.substring(0, name.length + 1) == (name + '=')) {
+                    cookieValue = decodeURIComponent(cookie.substring(name.length + 1));
+                    break;
+                }
+            }
+        }
+        return cookieValue;
+    }
+    function sameOrigin(url) {
+        // url could be relative or scheme relative or absolute
+        var host = document.location.host; // host + port
+        var protocol = document.location.protocol;
+        var sr_origin = '//' + host;
+        var origin = protocol + sr_origin;
+        // Allow absolute or scheme relative URLs to same origin
+        return (url == origin || url.slice(0, origin.length + 1) == origin + '/') ||
+            (url == sr_origin || url.slice(0, sr_origin.length + 1) == sr_origin + '/') ||
+            // or any other URL that isn't scheme relative or absolute i.e relative.
+            !(/^(\/\/|http:|https:).*/.test(url));
+    }
+    function safeMethod(method) {
+        return (/^(GET|HEAD|OPTIONS|TRACE)$/.test(method));
+    }
+
+    if (!safeMethod(settings.type) && sameOrigin(settings.url)) {
+        xhr.setRequestHeader("X-CSRFToken", getCookie('csrftoken'));
+    }
+});
+
+
 function scrollbarWidth() {
     return cachedScrollbarWidth;
 }
@@ -139,15 +179,18 @@ function calculateHiddenStuffHeight() {
 
 function stretchOneSidebarBox() {
     /* Stretch out one sidebarbox so that the sidebar is completely filled. */
-    var stillAvailable, includeMargin, minHeight, autoHeight, newHeight;
+    var minHeight, stillAvailable, newHeight, sidebarboxStretched,
+        includeMargin;
     minHeight = 100; // not smaller than this
-    $("#sidebar .sidebarbox-stretched").height('auto');
-    autoHeight = $("#sidebar .sidebarbox-stretched").height();
-    stillAvailable = $("#sidebar").innerHeight();
-    $("#sidebar > *").each(function () {
-        stillAvailable -= $(this).outerHeight(includeMargin = true);
+    newHeight = $("#sidebar").height();  // Start with total available
+    // Subtract other box sizes.
+    $("#sidebar > *").not(".sidebarbox-stretched").each(function () {
+        newHeight -= $(this).outerHeight(includeMargin = true);
     });
-    newHeight = autoHeight + stillAvailable;
+    // Now remove margin from sidebarbox-stretched, 1px
+    sidebarboxStretched = $("#sidebar .sidebarbox-stretched");
+    newHeight -= ($(sidebarboxStretched).outerHeight(includeMargin = true) -
+                  $(sidebarboxStretched).height());
     newHeight = newHeight < minHeight ? minHeight : newHeight;
     $("#sidebar .sidebarbox-stretched").height(newHeight);
 }
@@ -471,6 +514,7 @@ function setUpScreen() {
     stretchOneSidebarBox();
 }
 
+
 function setUpPortalTabs() {
     var selected_tab;
     selected_tab = $("#portal-tab-selected").attr("data-selected");
@@ -481,6 +525,42 @@ function setUpPortalTabs() {
     $(selected_tab).addClass("selected");
 }
 
+
+// Initialize Tipsy
+function setUpTipsy() {
+    $('a[rel=tipsy]').tipsy({
+        delayIn: 200,
+        delayOut: 200,
+        fade: false,
+        gravity: 'w'
+    });
+    $('a#logo-img').tipsy({
+        delayIn: 200,
+        delayOut: 200,
+        fade: false,
+        gravity: 'n'
+    });
+    $('span[rel=tipsy]').tipsy({
+        delayIn: 200,
+        delayOut: 200,
+        fade: false,
+        gravity: 's'
+    });
+    $('#summary-datepicker-a').tipsy({
+        delayIn: 200,
+        delayOut: 200,
+        fade: false,
+        gravity: 's'
+    });
+    $('[rel=tipsy-south]').tipsy({
+        delayIn: 200,
+        delayOut: 200,
+        fade: false,
+        gravity: 's'
+    });
+}
+
+
 function restretchExistingElements() {
     fillScreen();
     divideVerticalSpaceEqually();
@@ -517,39 +597,7 @@ $(document).ready(function () {
     setUpOverlays();
     setUpLogin();
     setUpPrintButton();
-
-    // Initialize Tipsy
-    $('a[rel=tipsy]').tipsy({
-        delayIn: 200,
-        delayOut: 200,
-        fade: false,
-        gravity: 'w'
-    });
-    $('a#logo-img').tipsy({
-        delayIn: 200,
-        delayOut: 200,
-        fade: false,
-        gravity: 'n'
-    });
-    $('span[rel=tipsy]').tipsy({
-        delayIn: 200,
-        delayOut: 200,
-        fade: false,
-        gravity: 's'
-    });
-    $('#summary-datepicker-a').tipsy({
-        delayIn: 200,
-        delayOut: 200,
-        fade: false,
-        gravity: 's'
-    });
-    $('#transparency-slider').tipsy({
-        delayIn: 200,
-        delayOut: 200,
-        fade: false,
-        gravity: 's'
-    });
-
+    setUpTipsy();
 
     //setUpWorkspaceAcceptableButtons();
     // Light up the selected tab, if available.
